@@ -1,7 +1,7 @@
 # Automated tests for the `executor' module.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 6, 2015
+# Last Change: October 18, 2015
 # URL: https://executor.readthedocs.org
 
 """Automated tests for the `executor` package."""
@@ -198,6 +198,29 @@ class ExecutorTestCase(unittest.TestCase):
         finally:
             os.rmdir(directory)
 
+    def test_virtual_environment_option(self):
+        """Make sure Python virtual environments can be used."""
+        directory = tempfile.mkdtemp()
+        virtual_environment = os.path.join(directory, 'environment')
+        try:
+            # Create a virtual environment to run the command in.
+            execute('virtualenv', virtual_environment)
+            # This is the expected value of `sys.executable'.
+            expected_executable = os.path.join(virtual_environment, 'bin', 'python')
+            # Get the actual value of `sys.executable' by running a Python
+            # interpreter inside the virtual environment.
+            actual_executable = execute('python', '-c', 'import sys; print(sys.executable)',
+                                        capture=True, virtual_environment=virtual_environment)
+            # Make sure the values match.
+            assert os.path.samefile(expected_executable, actual_executable)
+            # Make sure that shell commands are also supported (command line
+            # munging inside executor is a bit tricky and I specifically got
+            # this wrong on the first attempt :-).
+            output = execute('echo $VIRTUAL_ENV', capture=True, virtual_environment=virtual_environment)
+            assert os.path.samefile(virtual_environment, output)
+        finally:
+            shutil.rmtree(directory)
+
     def test_fakeroot_option(self):
         """Make sure ``fakeroot`` can be used."""
         filename = os.path.join(tempfile.gettempdir(), 'executor-%s-fakeroot-test' % os.getpid())
@@ -383,6 +406,16 @@ class ExecutorTestCase(unittest.TestCase):
         # Make sure all commands finished.
         assert all(cmd.is_finished for id, cmd in pool.commands)
 
+    def test_command_pool_delay_checks_noop(self):
+        """Make sure command pools with delayed error checking don't raise when ``check=False``."""
+        pool = CommandPool(delay_checks=True)
+        # Include a command that fails immediately.
+        pool.add(ExternalCommand('exit 1', check=False))
+        # Run the command pool without catching exceptions; we don't except any.
+        pool.run()
+        # Make sure the command failed even though the exception wasn't raised.
+        assert all(cmd.failed for id, cmd in pool.commands)
+
     def test_command_pool_logs_directory(self):
         """Make sure command pools can log output of commands in a directory."""
         root_directory = tempfile.mkdtemp()
